function [y,d]=aitken(x,theta)
% y=aitken(x)        Aitken's delta-squared transformation
% y=aitken(x,theta)  (theta~=0) theta-modified Aitken transformation
%      If the sequence converges like s+n^(-k), choose theta=1+1/k.
% y=aitken(x,0)      variable theta-modified Aitken transformation.
%     This algorithm is similar in behaviour to Brezinski's theta algorithm.
% [y,s]=aitken(a,m)  Aitken's transformation for series.  The first m terms 
%     are not transformed.  Default: m=1.  If this function is iterated, 
%     m should be incremented by 2 between calls.  If m<=0, this function
%     will be iterated for m=1,3,5,... as far as possible.
% TO DO: constant and variable theta-modified transformations for series.
%   DPL 2004-05-12

if nargout>1, d=x; x=cumsum(x); 
  if nargin<2, m=1; else m=theta; end
else d=diff(x); 
end

n=length(d); dd=diff(d);
if nargout>1, 
  if m<=0, for m=1:2:n-1, [y,d]=aitken(d,m); end; return; end;
  k=m+2:n; dm1=-d(m)*d(m+1)/dd(m); 
  d(k)=d(k-1).*(d(k-2)./dd(k-2)-d(k-1)./dd(k-1));
%  d(k)=d(k-1).*(d(k-2).*d(k)-d(k-1).^2)./(dd(k-2).*dd(k-1));
%  d(k)=d(k-1).*(d(k-2).*dd(k-1)-d(k-1).*dd(k-2))./(dd(k-2).*dd(k-1));
%  There is little difference between the above three formulas, but the
%  following formula is distinctly worse for divergent series.
%  d(k)=d(k)-(d(k).^2./dd(k-1)-d(k-1).^2./dd(k-2));
  d(m+1)=dm1; 
  y=cumsum(d);
  return
  end
  
i=2:n; a=-d(i-1).*d(i)./dd;
if nargin<2, theta=1; end
if theta~=0, y=x(i)+theta*a; 
else j=2:n-1; theta=-d(j)./diff(a); y=x(j)+theta.*a(j-1); end
