function [y,err,T]=romberg(func,a,b,tol,atleast)
% Romberg integration
% [y,T,err]=romberg(func,a,b,tol,atleast)
% y: best integral
% err: error estimate
% T: full Romberg table
% func: feval(func,x) must be posisble for a vector x
% a, b: limits
% tol: tolerance=[tolabs,tolrel]; (optional, set to [0,0])
% atleast: after so many steps, stop if no decrease in err  
%          optional, set to 10
if nargin<5, atleast=10; end
if nargin<4, tol=0; end
tol=abs(tol);
if max(tol)==0, tol=10*eps; end
if length(tol)<2, tol(2)=tol(1); end
tolabs=tol(1); tolrel=tol(2);
h=(b-a); T(1)=h/2*(feval(func,a)+feval(func,b)); k=1; err=1;
while 1,
  k=k+1; M=h*sum(feval(func,a+h/2:h:b)); h=h/2; s=4; T(k,1)=(M+T(k-1,1))/2;
  for j=2:k, T(k,j)=T(k,j-1)+(T(k,j-1)-T(k-1,j-1))/(s-1); s=s*4;
  end
  y=T(k,k); olderr=err; err=abs(y-T(k-1,k-1));
  if err<=max(tolabs,abs(tolrel*y)), break, end;
  if k>=atleast && err>=olderr, break, end
  end
end

